
# q-expansion of newform 154.2.f.e, downloaded from the LMFDB on 15 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 154
weight = 2
poly_data = [25, -25, -10, 5, 11, -7, 4, -3, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1, 0, 0, 0, 0, 0, 0], 1], [[-7800, 1065, 4949, 2103, -1896, 747, -714, 406], 1355], [[-10265, 1055, 5129, 2297, -1924, 698, -776, 420], 1355], [[-15715, 2190, 8547, 3584, -3353, 1246, -1327, 728], 1355], [[18200, -2485, -9470, -4907, 4424, -1743, 1666, -857], 1355], [[-18960, 3235, 9520, 4991, -4492, 1624, -1623, 891], 1355], [[-20550, 2415, 9646, 5723, -4891, 1942, -1829, 955], 1355]]

hecke_ring_character_values = [[45, [1, 0, 0, 0, 0, 0, 0, 0]], [57, [0, 0, 0, 1, 0, 0, 0, 0]]]
aps_data = [[1, 0, -1, 1, -1, 0, 0, 0], [1, -2, -1, 1, -1, -1, 0, -1], [0, 1, 1, 0, 1, 0, 0, 2], [0, 0, 0, 1, 0, 0, 0, 0], [0, -1, -2, 0, -1, -2, 0, 0], [-2, -1, 4, -3, 5, 1, 0, 0], [-1, 1, 2, 0, 2, 2, -2, 2], [-2, 2, -2, 0, -1, 1, 2, 0], [-3, 1, 1, -1, 1, 2, 0, 1], [-4, -1, 0, -1, 4, 0, 1, 0], [2, -1, -2, 1, -1, 1, 0, 0], [1, 3, 0, 2, -1, 1, -2, -1], [0, -2, -1, 0, -1, -1, 2, -2], [-5, 2, 3, -3, 0, 0, 2, -2], [0, -2, 3, -7, 6, -1, 2, -2], [-2, 3, 6, -1, 3, 1, 2, 2], [1, 1, 0, 3, -1, -4, -5, 4], [0, -2, -2, 0, -7, -1, 1, -4], [-2, 4, 4, -4, 1, 2, 3, -2], [8, -1, -9, 0, 2, 1, -1, -2], [5, -1, 0, 12, -5, -4, -3, 4], [-8, 3, 6, -5, 3, -3, 0, 0], [1, 2, 1, 0, -4, -2, 2, 4], [-1, 2, 5, -5, 2, 4, 0, 2], [0, -2, 1, 1, 3, -4, -3, -3], [-6, -1, 8, -7, 9, 1, 0, 0], [3, -2, 0, 9, -3, -5, -3, 5], [4, -8, -8, 7, -7, -4, 0, -4], [5, -5, -4, 4, -2, -4, -3, 1], [-5, 6, 0, -7, 5, 3, 4, 1], [-6, 2, 8, 0, 3, 1, -1, 4], [3, -5, -3, 3, -2, -4, -3, 1], [7, -1, -8, 0, -1, 2, -2, -2], [4, -2, 0, 10, -4, 0, 2, 0], [-10, 3, 13, 0, -3, -2, 2, 6], [-4, 4, 6, -8, 6, 2, 4, 0], [-7, 4, 12, -11, 6, 2, 10, -3], [-4, -6, 5, -8, 11, 2, -2, -2], [5, -3, -13, 1, -10, 5, 1, 1], [1, 4, 2, 5, -2, 2, -6, 5], [-3, 0, 1, 2, -5, 0, 6, -3], [-2, -2, 0, 0, 2, -6, 6, -4], [-11, 0, 0, -5, 11, 3, 3, -3], [8, 1, -7, 0, -4, -1, 1, 2], [4, 2, -3, 3, 1, 2, 1, 0], [-8, 2, -4, 4, 2, 4, 0, 2], [-2, 5, 9, 2, 4, -3, 1, 1], [9, -10, 1, 12, -8, -5, -8, -1], [5, -1, 0, 7, -5, -6, -5, 6], [7, -5, -13, 5, -8, -1, -3, -3], [0, 8, 5, 7, -3, -6, 1, 1], [-6, 2, 3, 1, -6, 1, 10, -4], [12, 4, -7, 7, 2, 4, 2, 0], [12, -2, -12, 14, -10, -6, -4, -4], [6, -2, 0, 4, -6, 3, 5, -3], [0, 4, 7, -7, 7, 14, -3, 10], [4, 3, -1, 0, -17, 0, 0, 6], [1, 4, 0, 9, -1, 5, 1, -5], [-6, -2, 12, -8, 14, 2, 0, 0], [1, -5, -6, 0, 4, 3, -3, -10], [-1, 0, -4, -13, 12, 0, 2, -1], [-3, 0, 0, -17, 3, 1, 1, -1], [-16, 7, 7, -7, 6, 12, 1, 5], [-1, 8, 6, 1, 2, 4, -6, 7], [-9, 0, 9, 0, 13, 8, -8, 0], [0, -3, -12, 1, -9, -5, -4, -4], [-7, 0, 2, -2, 3, 6, -3, 6], [5, -9, 0, -5, -5, -4, 5, 4], [-5, -5, 0, 0, -4, -4, 4, -10], [12, -16, -16, 8, -4, -8, -8, -4], [5, 3, -4, 4, 3, 6, 0, 3], [-11, 2, 0, -3, 11, 1, -1, -1], [-20, 5, 0, -23, 20, 8, 3, -8], [9, 3, -23, 23, 1, 2, 2, -1], [-1, 6, 7, 0, -7, -1, 1, 12], [17, -6, -11, 12, -5, 4, -1, -1], [-1, -1, 0, -22, 1, -3, -2, 3], [-9, -1, 11, -11, 1, 2, -2, 3], [3, -2, -5, 0, -21, 1, -1, -4], [-2, 1, 18, -5, 17, 7, 4, 4], [2, 11, 4, -4, 5, 10, 6, -1], [-5, 4, 4, -3, 0, 2, 6, -1], [-2, 9, 8, 5, -1, -5, 2, 2], [4, 3, 0, 10, -4, 1, -2, -1], [-2, -2, 12, -12, -4, -8, 2, -6], [7, -12, 6, -8, 9, -6, -2, -5], [-34, 15, 27, -29, 12, 5, 10, 10], [-5, 3, 8, 0, -9, -6, 6, 6], [12, -12, -5, 5, -7, -14, -5, -2], [-17, 7, 26, -26, 4, 8, 3, 1], [-4, 1, 5, 0, 0, -7, 7, 2], [-10, -2, 8, 0, 4, 0, 0, -4], [-7, 20, 12, 3, 0, 10, -6, 13], [-11, -7, 0, -14, 11, 4, 11, -4], [-6, -4, 0, -6, 6, 1, 5, -1], [-3, 12, 24, -23, 26, 6, -6, 9], [8, -8, -6, -10, 14, -4, -8, 0], [5, 2, 0, -2, -5, -8, -10, 8], [17, 5, -12, 0, 4, -4, 4, 10], [8, -10, -14, 10, -4, -14, -12, -12], [14, -10, -16, 6, 3, -5, -18, 4], [-14, 3, 0, 9, 14, 2, -1, -2], [-2, -8, -21, -1, -13, -10, -9, -9], [-1, 8, -5, 8, -5, 4, -6, 7], [11, -15, -11, 11, -7, -14, -8, 1], [-19, -5, 14, 0, 1, -7, 7, -10], [-12, 1, 0, -20, 12, 15, 14, -15], [-9, 5, -5, 5, 6, 12, -1, 7], [16, -1, -17, 0, -4, 3, -3, -2], [22, 0, 0, 34, -22, -1, -1, 1], [-13, 19, 25, -9, 6, 11, 15, 15], [-14, 10, 15, 1, -10, 5, 18, -4], [22, 10, 6, -6, 7, 14, 3, 4], [4, -8, 1, 1, -1, -4, 0, -4], [-1, -4, 0, -5, 1, 7, 11, -7], [23, -14, -19, 19, -3, -7, -32, 9], [7, -6, -13, 0, 9, -5, 5, -12], [-6, 3, 14, -7, 11, 5, 4, 4], [-4, -2, 0, -4, 4, -8, -6, 8], [-1, 15, 6, -6, 3, 6, 12, -9], [4, -2, -7, 7, -3, -6, 1, -4], [-5, 10, 14, -8, 4, 16, 13, 13], [-8, -2, 6, 0, 10, -2, 2, -4], [-8, 0, -5, 5, 7, 14, -7, 14], [12, -10, -23, 4, -13, 6, -2, -2], [2, -4, -22, 38, -38, -2, 0, -2], [-6, -6, 0, 0, 3, 5, -5, -12], [3, 10, 0, 19, -3, 1, -9, -1], [-9, -7, -21, 21, -5, -10, -2, -3], [9, -3, 0, -6, -9, -3, 0, 3], [-15, 9, 12, -11, 3, 1, 5, 5], [12, -5, -17, 0, -2, -1, 1, -10], [-2, 4, -4, -12, 12, 2, 0, 2], [-1, 11, 27, 3, 16, 3, 7, 7], [-23, 13, 22, -17, 9, 1, 7, 7], [13, 1, -14, 14, 2, 4, -1, 3], [10, -8, -4, -10, 16, -4, -12, 2], [-1, 12, 13, 0, 17, 10, -10, 24], [-24, -4, 20, 0, 24, -8, 8, -8], [-15, 1, 16, 0, 10, 0, 0, 2], [0, 0, 16, -11, 11, 0, 0, 0], [-2, 6, 0, -20, 2, 4, -2, -4], [8, -1, 22, 7, 23, 1, 0, 0], [11, -27, -26, 1, 1, -7, -17, -17], [-2, -1, 0, -5, 2, 6, 7, -6], [1, -2, -1, 18, -18, -1, 0, -1], [18, -11, -29, 0, -12, -9, 9, -22], [-18, 13, 27, -27, 6, 12, 7, -1], [-26, 24, 22, -22, 17, 34, 7, 10], [4, 9, 5, 0, 13, -4, 4, 18], [3, 12, 6, 7, 2, 6, -18, 15], [8, -15, 1, -1, -10, -20, -5, -5], [8, -18, -5, -20, 19, -9, 2, -10], [-29, 2, 0, -1, 29, -3, -5, 3], [-3, -2, 1, 0, -3, 2, -2, -4], [-25, 11, 19, -17, 8, -5, 3, 3], [2, 1, -1, 0, 2, 9, -9, 2], [-1, 8, 9, 0, -10, 5, -5, 16], [-13, -22, -10, -24, 12, 0, -11, -11], [-24, 7, 31, 0, 11, 4, -4, 14], [6, -8, -13, 13, -4, -8, -4, 0], [17, 5, 0, -6, -17, 4, -1, -4], [-26, 12, 37, -37, 1, 2, 11, -10], [2, 6, -1, 9, -4, 3, -10, 8], [-8, 10, -4, -12, -14, 18, 14, 14], [-2, 0, 0, -14, 2, 16, 16, -16], [-14, 6, -14, 14, -4, -8, 10, -14], [9, -1, 0, -10, -9, 1, 2, -1]]
